@@ -1,22 +1,49 @@
 /*
-    PGExt - pygame extension - color
-    Author: Josef Vanžura
-    Required: Python 2.5+, PyGame 1.8.1+, SDL 1.2
+    PYGAME EXTENSION - pgext
+    many parts of this project are taken from opensource projects:
+        libSDL
+        pygame
+
+    Copyright (c) 2012-2013, Josef Vanzura <gindar@zamraky.cz>
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright notice, this
+       list of conditions and the following disclaimer.
+    2. Redistributions in binary form must reproduce the above copyright notice,
+       this list of conditions and the following disclaimer in the documentation
+       and/or other materials provided with the distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+    The views and conclusions contained in the software and documentation are those
+    of the authors and should not be interpreted as representing official policies,
+    either expressed or implied, of the FreeBSD Project.
 */
 
 #include "pgext.h"
-
-#define PGEXT_VERSION 6
+#include "hsvlrgb.h"
 
 /*
     Set all opaque (alpha>0) pixels to specified color (R,G,B).
 */
-static PyObject* setColor( PyObject* self, PyObject* args )
+static PyObject*
+setColor( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,x,y = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
     PyObject *py_color = NULL;
     Uint8 color[4];
@@ -26,18 +53,16 @@ static PyObject* setColor( PyObject* self, PyObject* args )
         "OO", &py_surface, &py_color
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
     if(!RGBAFromColorObj (py_color, color))
         return Py_None;
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = getPixel(surface, x, y);
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a);
             if(a==0) continue;
-            pixel = SDL_MapRGBA( format, color[0], color[1], color[2], a );
-            setPixel(surface, x, y, pixel);
+            setPixel_RGBA(surface, x, y, color[0], color[1], color[2], a );
         }
     }
 
@@ -51,12 +76,12 @@ static PyObject* setColor( PyObject* self, PyObject* args )
         1 alpha = alpha
         2 alpha = alpha * (shift/100)
 */
-static PyObject* setAlpha( PyObject* self, PyObject* args )
+static PyObject*
+setAlpha( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,x,y = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
     int shift;
     int method = 0;
@@ -67,7 +92,7 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     if( method == 1 ){
         shift = clampUint8( shift );
     }
@@ -77,8 +102,7 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = getPixel(surface, x, y);
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
             if(method == 0){
                 a = a + shift;
                 a = clampUint8( a );
@@ -89,8 +113,7 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
             if(method == 2){
                 a = a * percent;
             }
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            setPixel(surface, x, y, pixel);
+            setPixel_RGBA(surface, x, y, r, g, b, a );
         }
     }
 
@@ -105,7 +128,8 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
         1 - HSV (saturation=0)
         2 - HSL (saturation=0)
 */
-static PyObject* greyscale( PyObject* self, PyObject* args )
+static PyObject*
+greyscale( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_PixelFormat *format = NULL;
@@ -121,7 +145,7 @@ static PyObject* greyscale( PyObject* self, PyObject* args )
         "O|i", &py_surface, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
     for(x = 0; x < w; x ++){
         for(y = 0; y < h; y ++){
@@ -148,7 +172,8 @@ static PyObject* greyscale( PyObject* self, PyObject* args )
 /*
     Invert surface colors
 */
-static PyObject* invert( PyObject* self, PyObject* args )
+static PyObject*
+invert( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -161,7 +186,7 @@ static PyObject* invert( PyObject* self, PyObject* args )
         "O", &py_surface
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     while( pxcnt-- ){
@@ -178,7 +203,8 @@ static PyObject* invert( PyObject* self, PyObject* args )
     method: (only 0)
         0 - hue += shift
 */
-static PyObject* hue( PyObject* self, PyObject* args )
+static PyObject*
+hue( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -194,7 +220,7 @@ static PyObject* hue( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     while( pxcnt-- ){
@@ -231,7 +257,8 @@ static PyObject* hue( PyObject* self, PyObject* args )
         1 - sat = s_shift
         2 - sat =  sat * (s_shift/100.0)
 */
-static PyObject* saturation( PyObject* self, PyObject* args )
+static PyObject*
+saturation( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -248,7 +275,7 @@ static PyObject* saturation( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
@@ -282,7 +309,8 @@ static PyObject* saturation( PyObject* self, PyObject* args )
         1 - val = v_shift
         2 - val =  val * (v_shift/100.0)
 */
-static PyObject* value( PyObject* self, PyObject* args )
+static PyObject*
+value( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -300,7 +328,7 @@ static PyObject* value( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
@@ -326,7 +354,8 @@ static PyObject* value( PyObject* self, PyObject* args )
     return Py_None;
 }
 
-static PyObject* lightness( PyObject* self, PyObject* args )
+static PyObject*
+lightness( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -343,7 +372,7 @@ static PyObject* lightness( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
@@ -371,12 +400,12 @@ static PyObject* lightness( PyObject* self, PyObject* args )
 }
 
 /* Color multiply */
-static PyObject* multiply( PyObject* self, PyObject* args )
+static PyObject*
+multiply( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,pxcnt = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
     float shift = 1.0;
     float rp = 1.0, gp = 1.0, bp = 1.0;
@@ -387,7 +416,7 @@ static PyObject* multiply( PyObject* self, PyObject* args )
         "Of|bbb", &py_surface, &shift, &rb, &gb, &bb
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if(rb){rp = shift;}
@@ -395,15 +424,13 @@ static PyObject* multiply( PyObject* self, PyObject* args )
     if(bb){bp = shift;}
 
     while( pxcnt-- ){
-        pixel = getPixelN(surface, pxcnt);
-        SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+        getPixelN_RGBA(surface, pxcnt, &r, &g, &b, &a);
 
         r = clampUint8(r * rp);
         g = clampUint8(g * gp);
         b = clampUint8(b * bp);
 
-        pixel = SDL_MapRGBA( format, r, g, b, a );
-        setPixelN(surface, pxcnt, pixel);
+        setPixelN_RGBA(surface, pxcnt, r, g, b, a);
     }
 
     return Py_None;
@@ -413,7 +440,8 @@ static PyObject* multiply( PyObject* self, PyObject* args )
 /*
     Replace pixels (hue == check_hue) with new_hue.
 */
-static PyObject* colorize( PyObject* self, PyObject* args )
+static PyObject*
+colorize( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -433,7 +461,7 @@ static PyObject* colorize( PyObject* self, PyObject* args )
         &py_surface, &check_hue, &new_hue, &v_shift, &s_shift
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
@@ -468,7 +496,8 @@ static PyObject* colorize( PyObject* self, PyObject* args )
 /*
     Apply per-pixel alpha mask on surface
 */
-static PyObject* alphaMask( PyObject* self, PyObject* args )
+static PyObject*
+alphaMask( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -489,8 +518,8 @@ static PyObject* alphaMask( PyObject* self, PyObject* args )
         &py_surface, &py_surface_mask, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface_mask, &mw, &mh, &format_mask, &surface_mask);
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface_mask, &mw, &mh, &format_mask, &surface_mask);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     target_surface = createSurface( mw, mh );
 
 
@@ -562,8 +591,7 @@ static PyMethodDef ColorMethods[] = {
 
 PyMODINIT_FUNC initcolor(void)
 {
-    PyObject *module = Py_InitModule("color", ColorMethods);
+    Py_InitModule("color", ColorMethods);
     import_pygame_surface();
     import_pygame_color();
-    PyObject_SetAttrString(module, "version", PyLong_FromLong(PGEXT_VERSION));
 }
