@@ -1,23 +1,101 @@
 /*
-    PGExt - pygame extension - filter
-    Author: Josef Vanžura
-    Required: Python 2.5+, PyGame 1.8.1+, SDL 1.2
+    PYGAME EXTENSION - pgext
+    many parts of this project are taken from opensource projects:
+        libSDL
+        pygame
+
+    Copyright (c) 2012-2013, Josef Vanzura <gindar@zamraky.cz>
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright notice, this
+       list of conditions and the following disclaimer.
+    2. Redistributions in binary form must reproduce the above copyright notice,
+       this list of conditions and the following disclaimer in the documentation
+       and/or other materials provided with the distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+    The views and conclusions contained in the software and documentation are those
+    of the authors and should not be interpreted as representing official policies,
+    either expressed or implied, of the FreeBSD Project.
 */
 
 #include "pgext.h"
 
-#define PGEXT_VERSION 6
+static SDL_Surface*
+_shadow( SDL_Surface *surface, int radius, float opacity, int w, int h, int rw, int rh, int offset, Uint8 color[4] );
+
+static void
+_blur( SDL_Surface *source, SDL_Surface *target, int radius, char vertical );
+
+static SDL_Surface*
+_gradient( int w, int h, Uint8 fcolor[4], Uint8 tcolor[4], char vertical, float ratio );
+
+
+
+static SDL_Surface*
+_shadow( SDL_Surface *surface, int radius, float opacity, int w, int h, int rw, int rh, int offset, Uint8 color[4] )
+{
+    int x,y;
+    Uint8 r,g,b,a;
+    SDL_Surface *target_surface = NULL;
+    SDL_Surface *temp_surface = NULL;
+
+    /* Helper surfaces */
+    target_surface = createSurface( rw, rh );
+    temp_surface = createSurface( rw, rh );
+
+    /* Lock surfaces for direct pixel set */
+    SDL_LockSurface (temp_surface);
+    SDL_LockSurface (target_surface);
+    /* Set color & copy surface to target */
+    for( y = 0; y < h; y ++ ){
+        for( x = 0; x < w; x ++ ){
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+            if(a == 0) continue;
+            if(opacity < 1){
+                a = (Uint8) (((double)a) * opacity);
+            }
+            setPixel_RGBA(target_surface, x + offset, y + offset, color[0], color[1], color[2], a);
+        }
+    }
+    SDL_UnlockSurface (temp_surface);
+    SDL_UnlockSurface (target_surface);
+
+    /* target => temp with horizontal blur */
+    _blur( target_surface, temp_surface, radius, 0 );
+    /* temp => target with vertical blur */
+    _blur( temp_surface, target_surface, radius, 1 );
+
+    /* Temp must be free */
+    SDL_FreeSurface( temp_surface );
+    return target_surface;
+}
+
+
 
-#define MAX_SHADOW_VALUE 20
 
 /* simple horizontal/vertical blur */
-static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char vertical ){
+static void
+_blur( SDL_Surface *source, SDL_Surface *target, int radius, char vertical )
+{
     Uint8 r,g,b,a;
-    double sr,sg,sb,sa;
     int color_cnt = 0;
-    int x,y,ra,n,s,o1,o2,pxn;
-    //double r2;
-    int pxcnt = 0;
+    int x,y,ra,n,s,o1,o2;
+    Uint8 color[4];
+    double sumcolor[4];
 
     /* Get sdl_surface attributes */
     int w = source->w;
@@ -27,38 +105,45 @@ static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char ve
     s = (vertical)?h:w;
     o1 = (vertical)?1:0;
     o2 = (vertical)?0:1;
-    //r2 = radius*2+1;
 
     SDL_LockSurface(source);
     SDL_LockSurface(target);
     for( y = 0; y < h; y++ ){
         for( x = 0; x < w; x++ ){
-            color_cnt = 0;sr = 0;sg = 0;sb = 0;sa = 0;
+            color_cnt = 0;
+            sumcolor[0] = 0;
+            sumcolor[1] = 0;
+            sumcolor[2] = 0;
+            sumcolor[3] = 0;
             ra = -radius;
             n = (vertical)?y:x;
 
-            /* Sum of pixel within radius */
+            /* Sum of pixels within radius */
             while( ra <= radius ){
-                if( (n+ra) >= 0 && (n+ra) < s ){
-                    pxn = pxcnt + (int)( ((ra * o1) * w) + (ra * o2));
-                    getPixelN_RGBA(source, pxn, &r, &g, &b, &a );
-                    sr += r;sg += g;sb += b;sa += a;
+                if( (n + ra) == s ) break;
+                if( (n + ra) >= 0 ){
+                    getPixel_RGBA(source, x + (ra * o2), y + (ra * o1),
+                        &r, &g, &b, &a);
+                    sumcolor[0] += r;
+                    sumcolor[1] += g;
+                    sumcolor[2] += b;
+                    sumcolor[3] += a;
                     color_cnt++;
                 }
                 ra++;
             }
-            pxcnt++;
-            if(sa == 0) continue;
+            //if(sa == 0) continue;
+            if(sumcolor[3] == 0) continue;
             if(color_cnt <= 1) continue;
 
             /* average channel value */
-            sr = sr / color_cnt;
-            sg = sg / color_cnt;
-            sb = sb / color_cnt;
-            sa = sa / color_cnt;
+            color[0] = sumcolor[0] / color_cnt;
+            color[1] = sumcolor[1] / color_cnt;
+            color[2] = sumcolor[2] / color_cnt;
+            color[3] = sumcolor[3] / color_cnt;
 
             /* map & set pixel */
-            setPixel_RGBA(target, x, y, (Uint8)sr, (Uint8)sg, (Uint8)sb, (Uint8)sa);
+            setPixel_RGBA(target, x, y, color[0], color[1], color[2], color[3]);
         }
     }
     SDL_UnlockSurface(source);
@@ -66,15 +151,61 @@ static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char ve
 }
 
 
+static SDL_Surface*
+_gradient( int w, int h, Uint8 fcolor[4], Uint8 tcolor[4], char vertical, float ratio )
+{
+    SDL_Surface *surface = NULL;
+    SDL_PixelFormat *format = NULL;
+    Uint32 pixel;
+    Uint8 color[4];
+    int dcolor[4];
+    int x,y,i,q,rows,cols;
+    double nn,delta;
+
+
+    dcolor[0] = (int)(tcolor[0]-fcolor[0]);
+    dcolor[1] = (int)(tcolor[1]-fcolor[1]);
+    dcolor[2] = (int)(tcolor[2]-fcolor[2]);
+    dcolor[3] = (int)(tcolor[3]-fcolor[3]);
+
+    surface = createSurface( w, h );
+    format = surface->format;
+
+    rows = vertical?h:w;
+    cols = vertical?w:h;
+
+    SDL_LockSurface (surface);
+    for( i = 0; i < rows; i ++ ){
+        delta = ((i+1)/(float)(rows+1));
+        nn = pow( delta, ratio );
+
+        color[0] = (Uint8)( fcolor[0] + ( dcolor[0] * nn ) );
+        color[1] = (Uint8)( fcolor[1] + ( dcolor[1] * nn ) );
+        color[2] = (Uint8)( fcolor[2] + ( dcolor[2] * nn ) );
+        color[3] = (Uint8)( fcolor[3] + ( dcolor[3] * nn ) );
+
+        pixel = SDL_MapRGBA( format, color[0], color[1], color[2], color[3] );
+        for( q = 0; q < cols; q ++ ){
+            x = vertical?q:i;
+            y = vertical?i:q;
+            setPixel(surface, x, y, pixel);
+        }
+    }
+    SDL_UnlockSurface (surface);
+    return surface;
+}
+
+
+
 /*
     Simple random noise. [RGB]+RAND
 */
-static PyObject* noise( PyObject* self, PyObject* args )
+static PyObject*
+noise( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,x,y = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
 
     int cnt = 0;
@@ -87,7 +218,7 @@ static PyObject* noise( PyObject* self, PyObject* args )
         "O|ii", &py_surface, &value, &density
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     //srand();
 
     cnt = (w * h) / density;
@@ -96,8 +227,7 @@ static PyObject* noise( PyObject* self, PyObject* args )
         x = (rand() % w);
         y = (rand() % h);
 
-        pixel = getPixel(surface, x, y);
-        SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+        getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
 
         rnd = (value / 2) - (rand() % value);
         tr = r + rnd;
@@ -107,32 +237,12 @@ static PyObject* noise( PyObject* self, PyObject* args )
         g = clampUint8(tg);
         b = clampUint8(tb);
 
-        pixel = SDL_MapRGBA( format, r, g, b, a );
-        setPixel(surface, x, y, pixel);
+        setPixel_RGBA(surface, x, y, r, g, b, a);
 
         cnt--;
         if(cnt < 0) break;
     }
 
-/*    for( y = 0; y < h; y ++ ){
-        for( x = 0; x < w; x ++ ){
-            if( rand()%density != 0 ) continue;
-            pixel = getPixel(surface, x, y);
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-
-            rnd = (value / 2) - (rand() % value);
-            tr = r + rnd;
-            tg = g + rnd;
-            tb = b + rnd;
-            r = clampUint8(tr);
-            g = clampUint8(tg);
-            b = clampUint8(tb);
-
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            setPixel(surface, x, y, pixel);
-        }
-    }
-*/
     return Py_None;
 }
 
@@ -140,7 +250,8 @@ static PyObject* noise( PyObject* self, PyObject* args )
 /*
     noise blur
 */
-static PyObject* noiseBlur( PyObject* self, PyObject* args )
+static PyObject*
+noiseBlur( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -156,7 +267,7 @@ static PyObject* noiseBlur( PyObject* self, PyObject* args )
         "O|ib", &py_surface, &radius, &blend
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     //srand();
 
     for( y = 0; y < h; y ++ ){
@@ -183,7 +294,8 @@ static PyObject* noiseBlur( PyObject* self, PyObject* args )
 /*
     Simple image scratching.
 */
-static PyObject* scratch( PyObject* self, PyObject* args )
+static PyObject*
+scratch( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -200,7 +312,7 @@ static PyObject* scratch( PyObject* self, PyObject* args )
         "O|i", &py_surface, &offset
     PY_PARSEARGS_END
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
     surface_copy = SDL_ConvertSurface(surface, format, 0);
 
@@ -224,7 +336,8 @@ static PyObject* scratch( PyObject* self, PyObject* args )
 /*
     Simple image scratching.
 */
-static PyObject* pixelize( PyObject* self, PyObject* args )
+static PyObject*
+pixelize( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
@@ -241,7 +354,7 @@ static PyObject* pixelize( PyObject* self, PyObject* args )
 
     halfoffset = (int) (((float) offset) / 2);
 
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
@@ -263,21 +376,17 @@ static PyObject* pixelize( PyObject* self, PyObject* args )
 /*
     Horizontal/vertical gradient.
 */
-static PyObject* gradient( PyObject* self, PyObject* args )
+static PyObject*
+gradient( PyObject* self, PyObject* args )
 {
 
     SDL_Surface *surface = NULL;
     PyObject *py_fcolor = NULL, *py_tcolor = NULL;
-    SDL_PixelFormat *format = NULL;
-
-    Uint32 pixel;
-    Uint8 color[4];
     Uint8 fcolor[4];
     Uint8 tcolor[4];
-    int x,y,i,q,w,h,dR,dG,dB,dA,rows,cols;
+    int w,h;
     char vertical = 0;
     float ratio = 0.0;
-    double nn,delta;
 
 
     PY_PARSEARGS_BEGIN
@@ -290,35 +399,7 @@ static PyObject* gradient( PyObject* self, PyObject* args )
     if(!RGBAFromColorObj (py_tcolor, tcolor))
         return Py_None;
 
-    dR = (int)(tcolor[0]-fcolor[0]);
-    dG = (int)(tcolor[1]-fcolor[1]);
-    dB = (int)(tcolor[2]-fcolor[2]);
-    dA = (int)(tcolor[3]-fcolor[3]);
-
-    surface = createSurface( w, h );
-    format = surface->format;
-
-    rows = vertical?h:w;
-    cols = vertical?w:h;
-
-    SDL_LockSurface (surface);
-    for( i = 0; i < rows; i ++ ){
-        delta = ((i+1)/(float)(rows+1));
-        nn = pow( delta, ratio );
-
-        color[0] = (Uint8)( fcolor[0] + ( dR * nn ) );
-        color[1] = (Uint8)( fcolor[1] + ( dG * nn ) );
-        color[2] = (Uint8)( fcolor[2] + ( dB * nn ) );
-        color[3] = (Uint8)( fcolor[3] + ( dA * nn ) );
-
-        pixel = SDL_MapRGBA( format, color[0], color[1], color[2], color[3] );
-        for( q = 0; q < cols; q ++ ){
-            x = vertical?q:i;
-            y = vertical?i:q;
-            setPixel(surface, x, y, pixel);
-        }
-    }
-    SDL_UnlockSurface (surface);
+    surface = _gradient(w, h, fcolor, tcolor, vertical, ratio);
 
     return PySurface_New( surface );
 }
@@ -326,20 +407,15 @@ static PyObject* gradient( PyObject* self, PyObject* args )
 /*
     Create shadow from RGBA surface.
 */
-static PyObject* shadow(PyObject* self, PyObject* args)
+static PyObject*
+shadow(PyObject* self, PyObject* args)
 {
     PyObject *py_surface = NULL, *py_color = NULL;
     SDL_Surface *surface;
-    int w,h,x,y;
-    int rw,rh;
-    int offset = 0;
+    int w,h;
     SDL_PixelFormat *format;
-    Uint8 r,g,b,a;
-
-    SDL_Surface *target_surface = NULL;
-    SDL_Surface *temp_surface = NULL;
-
     Uint8 color[4];
+    SDL_Surface *target_surface = NULL;
 
     int radius = 5;
     float opacity = 1.0;
@@ -349,62 +425,32 @@ static PyObject* shadow(PyObject* self, PyObject* args)
         "OO|ibf", &py_surface, &py_color, &radius, &resized, &opacity
     PY_PARSEARGS_END
 
-    if(!RGBAFromColorObj (py_color, color))
-        return Py_None;
-
-    if( radius > MAX_SHADOW_VALUE )
-        radius = MAX_SHADOW_VALUE;
-
-    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    if(!RGBAFromColorObj (py_color, color)) return Py_None;
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
+    /* only 32bit surfaces are supported */
     if(format->BytesPerPixel == 3){
         return RAISE (PyExc_ValueError, "input surface must be 32bit surface not 24bit");
     }
 
     if( resized ){
         /* Set offsets for resized surface */
-        rw = w + (radius * 2);
-        rh = h + (radius * 2);
-        offset = radius;
+        target_surface = _shadow(surface, radius, opacity, w, h,
+            (w + (radius * 2)), (h + (radius * 2)),
+            radius, color );
     } else {
-        rw = w;
-        rh = h;
-        offset = 0;
+        target_surface = _shadow( surface, radius, opacity,
+            w, h, w, h, 0, color );
     }
 
-    /* Helper surfaces */
-    target_surface = createSurface( rw, rh );
-    temp_surface = createSurface( rw, rh );
-
-    /* Lock surfaces for direct pixel set */
-    SDL_LockSurface (temp_surface);
-    SDL_LockSurface (target_surface);
-    /* Set color & copy surface to target */
-    for( y = 0; y < h; y ++ ){
-        for( x = 0; x < w; x ++ ){
-            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
-            a = (Uint8) (((double)a)*opacity);
-            setPixel_RGBA(target_surface, x + offset, y + offset, color[0], color[1], color[2], a);
-        }
-    }
-    SDL_UnlockSurface (temp_surface);
-    SDL_UnlockSurface (target_surface);
-
-    /* Switch target => temp with horizontal blur */
-    _blur( target_surface, temp_surface, radius, 0 );
-    /* Switch temp => target with vertical blur */
-    _blur( temp_surface, target_surface, radius, 1 );
-
-    /* Temp must be free */
-    SDL_FreeSurface( temp_surface );
-
     return PySurface_New( target_surface );
 }
 
 /*
     Simple&fast blur
 */
-static PyObject* blur(PyObject* self, PyObject* args)
+static PyObject*
+blur(PyObject* self, PyObject* args)
 {
 
     PyObject* py_surface = NULL;
@@ -439,13 +485,12 @@ static PyObject* blur(PyObject* self, PyObject* args)
 /*
     Simple&fast blur
 */
-static PyObject* hvBlur(PyObject* self, PyObject* args)
+static PyObject*
+hvBlur(PyObject* self, PyObject* args)
 {
-
     PyObject* py_surface = NULL;
     PySurfaceObject *surface;
     SDL_Surface *copy_surface = NULL;
-    SDL_PixelFormat *format;
     unsigned int radius = 5;
     char vertical = 0;
 
@@ -455,10 +500,9 @@ static PyObject* hvBlur(PyObject* self, PyObject* args)
     PY_PARSEARGS_END
 
     surface = (PySurfaceObject*) py_surface;
-    format = (SDL_PixelFormat *)surface->surf->format;
 
     /* Kopie je potreba kvuli zdrojovym pixelum */
-    copy_surface = SDL_ConvertSurface(surface->surf, format, 0);
+    copy_surface = SDL_ConvertSurface(surface->surf, surface->surf->format, 0);
 
     /* Switch original => temp with vertical blur */
     _blur( copy_surface, surface->surf, radius, vertical );
@@ -468,6 +512,64 @@ static PyObject* hvBlur(PyObject* self, PyObject* args)
     return Py_None;
 }
 
+/*
+    Simple&fast ripple effect
+*/
+static PyObject*
+ripple(PyObject* self, PyObject* args)
+{
+    PyObject* py_surface = NULL;
+    int w,h,o,i,j,imax,jmax;
+    Uint32 pixel;
+    SDL_PixelFormat *format = NULL;
+
+    SDL_Surface *surface = NULL;
+    SDL_Surface *copy_surface = NULL;
+
+    int frequency = 10;
+    int amplitude = 5;
+    double phase = 0.0;
+    char vertical = 0;
+
+    double offset = 0;
+    double q;
+
+    /* python arguments parsing*/
+    PY_PARSEARGS_BEGIN
+        "Oii|db", &py_surface, &frequency, &amplitude, &phase, &vertical
+    PY_PARSEARGS_END
+
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
+
+    /* surface copy is needed for source pixels  */
+    copy_surface = SDL_ConvertSurface(surface, format, 0);
+
+    imax = vertical ? w : h;
+    jmax = vertical ? h : w;
+
+    for( i = 0; i < imax; i++ ){
+        q = ((double)(i % frequency)) / ((double) frequency);
+        offset = sin((q + phase) * 3.1415 * 2) * ((double)amplitude);
+        for( j = 0; j < jmax; j++ ){
+            o = (int) round_double(j + offset);
+            if(o < 0) o = jmax - o;
+            if(o >= jmax) o = o - jmax;
+            if(vertical){
+                pixel = getPixel(copy_surface, i, o);
+                setPixel(surface, i, j, pixel);
+            } else {
+                pixel = getPixel(copy_surface, o, i);
+                setPixel(surface, j, i, pixel);
+            }
+        }
+    }
+
+    SDL_FreeSurface( copy_surface );
+
+    return Py_None;
+}
+
+
 static PyMethodDef FiltersMethods[] = {
     {"gradient", gradient, METH_VARARGS, "Create vertical/horizontal gradient"},
     {"blur", blur, METH_VARARGS, "Blur surface"},
@@ -477,13 +579,13 @@ static PyMethodDef FiltersMethods[] = {
     {"shadow", shadow, METH_VARARGS, "Create RGBA surface shadow"},
     {"scratch", scratch, METH_VARARGS, "Create RGBA surface shadow"},
     {"pixelize", pixelize, METH_VARARGS, "Fast pixelize effect"},
+    {"ripple", ripple, METH_VARARGS, "Simple ripple effect"},
     {NULL, NULL, NULL, NULL}
 };
 
 PyMODINIT_FUNC initfilters(void)
 {
-    PyObject *module = Py_InitModule("filters", FiltersMethods);
+    Py_InitModule("filters", FiltersMethods);
     import_pygame_surface();
     import_pygame_color();
-    PyObject_SetAttrString(module, "version", PyLong_FromLong(PGEXT_VERSION));
 }
