@@ -1,14 +1,15 @@
 #!/bin/env python
 
 import sys
-import pgext
 import pygame
+import pgext
 import time
 
 pygame.display.init()
 pygame.display.set_mode((1, 1))
 
 
+
 class Tester:
     def __init__(self, source_image, num=1):
         self.num = num
@@ -61,11 +62,11 @@ class Tester:
             for test in self.order:
                 if single and test not in single:
                     continue
-                self.doTest(test)
+                self.doTest(test, n)
         self.saveAll()
 
-    def doTest(self, name):
-        print "%sTest: %s" % ("\b" * 80, name),
+    def doTest(self, name, n=1):
+        print "%sTest: %s/%s %s" % (("\b" * 80), n + 1, self.num,  name),
         try:
             test = self.tests[name]
             source = test[3].copy()
@@ -90,7 +91,7 @@ class Tester:
             self.results[name] = e
 
     def printResults(self):
-        print "Testing cycles: %s" % self.num
+        print "%sTesting cycles: %s" % (("\b" * 80), self.num)
         count = 0
         for test in self.order:
             if not test in self.results:
@@ -110,7 +111,7 @@ source_mask = pygame.image.load("source_mask.png")
 source_mask2 = pygame.image.load("source_mask2.png")
 source_colorize = pygame.image.load("source_colorize.png")
 
-tester = Tester("source_rgb.png", 1)
+tester = Tester("source_rgb.png", 30)
 
 tester.addTests([
                 ("color.colorize", "modify-inplace",
@@ -147,12 +148,13 @@ tester.addTests([
                 ("filters.shadow", "modify-return",
                  pgext.filters.shadow, [(0, 0, 0), 5, 1, 0.9], source_rgba),
                 ("filters.shadow2", "modify-return",
-                 pgext.filters.shadow, [(200, 0, 0), 5, 1, 0.9], source_rgba),
+                 pgext.filters.shadow, [(200, 0, 0), 10, 0, 0.9], source_rgba),
                 ("filters.blur", "modify-inplace", pgext.filters.blur, [4]),
                 ("filters.hvBlur-h", "modify-inplace",
                  pgext.filters.hvBlur, [6, 0]),
                 ("filters.hvBlur-v", "modify-inplace",
                  pgext.filters.hvBlur, [6, 1]),
+
                 ("filters.noise", "modify-inplace",
                  pgext.filters.noise, [55, 2]),
                 ("filters.noiseBlur1", "modify-inplace",
@@ -164,12 +166,16 @@ tester.addTests([
                 ("filters.pixelize", "modify-inplace",
                  pgext.filters.pixelize, [10]),
 
+                ("filters.ripple", "modify-inplace",
+                 pgext.filters.ripple, [30, 4]),
+
                 ("filters.gradient1", "make", pgext.filters.gradient,
                  [(200, 200), (100, 100, 200), (100, 200, 100), 0, 1.0]),
                 ("filters.gradient2", "make", pgext.filters.gradient,
                  [(200, 200), (255, 0, 0, 0), (0, 0, 255, 255), 0, 0.3]),
                 ("filters.gradient3", "make", pgext.filters.gradient,
                  [(200, 200), (100, 0, 0), (0, 55, 55), 1, 1.0]),
+
                 ])
 
 try:
